
# q-expansion of newform 2100.2.d.h, downloaded from the LMFDB on 11 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2100
weight = 2
poly_data = [1, 0, 3, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[1, 1, 1, 0], 1], [[0, 4, 0, 2], 1], [[-1, 1, -1, 0], 1]]

hecke_ring_character_values = [[1051, [1, 0, 0, 0]], [701, [-1, 0, 0, 0]], [1177, [1, 0, 0, 0]], [1501, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [1, 0, 0, -1], [0, 0, 0, 0], [1, -1, -1, 0], [0, 1, -1, 1], [0, -1, 0, -1], [2, 2, 0, -2], [0, 1, 3, 1], [0, -3, -2, -3], [0, 0, 2, 0], [0, -1, -1, -1], [-2, -2, 0, 2], [6, -2, 0, 2], [6, 1, 0, -1], [-2, -1, 0, 1], [0, -1, -4, -1], [-4, -4, 0, 4], [0, -4, 0, -4], [-6, -3, 0, 3], [0, -5, -3, -5], [0, -1, 4, -1], [8, 2, 0, -2], [-6, -1, 0, 1], [10, 2, 0, -2], [0, 5, 4, 5], [2, 4, 0, -4], [0, -1, 6, -1], [0, -3, 2, -3], [2, -4, 0, 4], [0, -1, -8, -1], [10, 9, 0, -9], [-8, -6, 0, 6], [0, 1, 0, 1], [0, 1, 7, 1], [0, 6, 4, 6], [4, 4, 0, -4], [0, 5, -4, 5], [-10, -7, 0, 7], [-10, 1, 0, -1], [2, 0, 0, 0], [0, 1, -7, 1], [0, 2, 2, 2], [0, -1, -3, -1], [6, 0, 0, 0], [0, 1, 12, 1], [0, -1, 5, -1], [-8, 0, 0, 0], [0, 3, 4, 3], [22, 1, 0, -1], [0, 8, 2, 8], [0, -5, 4, -5], [0, -5, 3, -5], [0, 10, 2, 10], [8, 10, 0, -10], [-14, -2, 0, 2], [0, 5, 8, 5], [-22, 2, 0, -2], [0, -9, -5, -9], [6, -6, 0, 6], [0, 6, 2, 6], [0, 7, 0, 7], [-14, 0, 0, 0], [0, -5, -6, -5], [20, 10, 0, -10], [0, 3, -4, 3], [0, 5, 0, 5], [-16, -4, 0, 4], [-2, 0, 0, 0], [0, -7, -12, -7], [0, 4, 2, 4], [18, 6, 0, -6], [0, -13, -9, -13], [0, 7, 8, 7], [-10, 2, 0, -2], [4, -2, 0, 2], [-14, -1, 0, 1], [0, -2, 0, -2], [0, -3, 0, -3], [0, 4, 12, 4], [0, 0, 6, 0], [28, 0, 0, 0], [30, 2, 0, -2], [0, -1, 1, -1], [0, 7, 12, 7], [0, -5, 1, -5], [0, 13, 4, 13], [0, 8, 10, 8], [14, -4, 0, 4], [22, 2, 0, -2], [22, -1, 0, 1], [-18, -15, 0, 15], [-8, -4, 0, 4], [2, 1, 0, -1], [0, 9, 15, 9], [-4, -10, 0, 10], [10, 7, 0, -7], [18, 4, 0, -4], [-2, 8, 0, -8], [0, 3, -6, 3], [-34, -8, 0, 8], [-14, -3, 0, 3], [0, 5, 4, 5], [2, 13, 0, -13], [0, 8, 6, 8], [-8, 12, 0, -12], [0, 13, 8, 13], [-10, 3, 0, -3], [-22, 2, 0, -2], [0, 7, 11, 7], [0, 14, 14, 14], [0, -13, -12, -13], [-10, -10, 0, 10], [0, -15, -4, -15], [0, -3, -1, -3], [4, 4, 0, -4], [0, -6, 10, -6], [0, 3, -14, 3], [-2, 9, 0, -9], [0, 11, 4, 11], [0, 1, -19, 1], [0, 12, 4, 12], [-10, -4, 0, 4], [-6, 0, 0, 0], [0, -3, 12, -3], [0, 5, 1, 5], [0, 4, -4, 4], [-18, -4, 0, 4], [16, -8, 0, 8], [0, 3, 0, 3], [0, 7, 20, 7], [28, 10, 0, -10], [0, -7, -8, -7], [-44, 0, 0, 0], [-18, -14, 0, 14], [-14, -14, 0, 14], [0, -6, -12, -6], [18, 0, 0, 0], [0, 3, -10, 3], [-14, -12, 0, 12], [0, -10, 0, -10], [0, -3, 1, -3], [0, -12, -16, -12], [-2, 11, 0, -11], [0, -3, -18, -3], [0, -10, -4, -10], [8, 4, 0, -4], [0, -9, -4, -9], [26, 2, 0, -2], [0, -7, 11, -7], [0, -7, -22, -7], [30, 2, 0, -2], [2, 12, 0, -12], [22, 11, 0, -11], [-10, -5, 0, 5], [26, 1, 0, -1], [0, -13, -15, -13], [-32, -2, 0, 2], [10, 8, 0, -8], [0, 1, -8, 1], [22, 18, 0, -18], [0, 1, 12, 1], [0, 3, 0, 3], [-38, -5, 0, 5], [8, -6, 0, 6], [0, 21, 12, 21], [-22, -15, 0, 15], [20, 16, 0, -16], [0, 13, 0, 13], [-2, -6, 0, 6], [42, 8, 0, -8], [0, -7, 9, -7], [0, -14, -6, -14], [0, -5, -15, -5], [-10, 0, 0, 0], [0, -5, 1, -5], [-18, -4, 0, 4], [32, -8, 0, 8], [0, -2, -26, -2], [0, -13, 4, -13], [0, 2, -8, 2], [-6, -13, 0, 13], [8, 14, 0, -14], [30, 6, 0, -6], [-22, 6, 0, -6], [0, -19, -12, -19], [-22, -4, 0, 4], [14, 2, 0, -2], [0, -1, -2, -1], [14, 16, 0, -16], [44, -2, 0, 2], [0, -5, -24, -5], [0, -7, 6, -7], [-24, 8, 0, -8], [-34, -8, 0, 8], [0, 5, 4, 5], [18, -14, 0, 14], [-18, -8, 0, 8], [-26, 2, 0, -2], [2, 14, 0, -14], [-38, -13, 0, 13], [0, 12, -14, 12], [0, -1, 23, -1], [0, -11, -16, -11], [0, -2, -24, -2], [-20, -4, 0, 4], [26, -12, 0, 12], [0, -5, -11, -5], [0, -3, -2, -3], [0, -6, -24, -6], [0, 5, -19, 5], [-34, -16, 0, 16], [-54, -2, 0, 2], [46, 13, 0, -13], [38, -7, 0, 7], [-8, -12, 0, 12], [0, 2, 34, 2], [-38, -3, 0, 3], [-22, -14, 0, 14], [0, -3, 0, -3], [0, 7, 12, 7], [-18, 10, 0, -10], [0, 19, 5, 19], [0, 2, -16, 2], [-50, 5, 0, -5], [-26, -9, 0, 9], [-6, -4, 0, 4], [10, 22, 0, -22], [0, -5, -1, -5], [0, 11, 6, 11], [0, 9, 11, 9], [-34, -18, 0, 18], [0, -19, -13, -19], [12, 12, 0, -12], [0, -6, -14, -6], [0, 23, 14, 23], [-42, 3, 0, -3], [0, -8, -14, -8], [0, -9, 16, -9], [0, -7, 17, -7], [-44, -10, 0, 10], [0, 13, 10, 13], [0, 1, -19, 1], [0, -5, 14, -5], [10, -6, 0, 6], [26, -6, 0, 6], [-56, -8, 0, 8], [0, 7, -4, 7], [8, 2, 0, -2], [-52, -10, 0, 10], [0, 1, -18, 1], [22, 22, 0, -22], [42, 2, 0, -2], [0, 9, 2, 9], [0, -32, -14, -32], [2, 12, 0, -12], [0, 13, 29, 13], [18, 26, 0, -26], [0, 11, 20, 11], [34, 0, 0, 0], [0, -11, 8, -11], [14, -1, 0, 1], [0, 13, -12, 13], [0, 8, -14, 8], [0, 7, -4, 7], [-14, -2, 0, 2], [0, -7, -29, -7], [0, -20, -10, -20], [30, 8, 0, -8], [-2, -1, 0, 1], [0, 11, 12, 11], [0, 10, 6, 10], [18, 11, 0, -11], [0, 7, -24, 7], [24, -6, 0, 6], [0, -11, -28, -11], [0, -13, -12, -13], [0, -3, 14, -3], [38, 20, 0, -20], [-14, 14, 0, -14], [-16, 10, 0, -10], [-30, -3, 0, 3], [-28, -8, 0, 8], [14, -13, 0, 13], [0, 10, 14, 10], [0, -5, -10, -5], [0, 3, -7, 3], [-2, 12, 0, -12], [0, 1, 12, 1], [0, 15, 5, 15], [-54, -24, 0, 24], [0, 16, 8, 16], [-58, -11, 0, 11], [0, 3, 12, 3], [16, -14, 0, 14], [6, -14, 0, 14], [-2, 16, 0, -16], [0, 23, -5, 23], [-38, 8, 0, -8], [-36, -28, 0, 28], [0, 3, -14, 3], [0, 15, 0, 15], [0, -3, -16, -3], [0, -10, 0, 10]]
